@@ -1,27 +1,30 @@
-import os, ssl, time, datetime, tempfile, shutil
+import os, ssl, time, datetime
 from pyasn1.type import univ, constraint, char, namedtype, tag
 from pyasn1.codec.der.decoder import decode
 from pyasn1.error import PyAsn1Error
 import OpenSSL
 import tcp
 
+default_exp = 62208000 # =24 * 60 * 60 * 720
+default_o = "mitmproxy"
+default_cn = "mitmproxy"
 
-def create_ca():
+def create_ca(o=default_o, cn=default_cn, exp=default_exp):
     key = OpenSSL.crypto.PKey()
     key.generate_key(OpenSSL.crypto.TYPE_RSA, 1024)
     ca = OpenSSL.crypto.X509()
     ca.set_serial_number(int(time.time()*10000))
     ca.set_version(2)
-    ca.get_subject().CN = "mitmproxy"
-    ca.get_subject().O = "mitmproxy"
+    ca.get_subject().CN = cn
+    ca.get_subject().O = o
     ca.gmtime_adj_notBefore(0)
-    ca.gmtime_adj_notAfter(24 * 60 * 60 * 720)
+    ca.gmtime_adj_notAfter(exp)
     ca.set_issuer(ca.get_subject())
     ca.set_pubkey(key)
     ca.add_extensions([
       OpenSSL.crypto.X509Extension("basicConstraints", True,
                                    "CA:TRUE"),
-      OpenSSL.crypto.X509Extension("nsCertType", True,
+      OpenSSL.crypto.X509Extension("nsCertType", False,
                                    "sslCA"),
       OpenSSL.crypto.X509Extension("extendedKeyUsage", True,
                                     "serverAuth,clientAuth,emailProtection,timeStamping,msCodeInd,msCodeCom,msCTLSign,msSGC,msEFS,nsSGC"
@@ -35,7 +38,7 @@ def create_ca():
     return key, ca
 
 
-def dummy_ca(path):
+def dummy_ca(path, o=default_o, cn=default_cn, exp=default_exp):
     dirname = os.path.dirname(path)
     if not os.path.exists(dirname):
         os.makedirs(dirname)
@@ -45,26 +48,26 @@ def dummy_ca(path):
     else:
         basename = os.path.basename(path)
 
-    key, ca = create_ca()
+    key, ca = create_ca(o=o, cn=cn, exp=exp)
 
     # Dump the CA plus private key
-    f = open(path, "w")
+    f = open(path, "wb")
     f.write(OpenSSL.crypto.dump_privatekey(OpenSSL.crypto.FILETYPE_PEM, key))
     f.write(OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, ca))
     f.close()
 
     # Dump the certificate in PEM format
-    f = open(os.path.join(dirname, basename + "-cert.pem"), "w")
+    f = open(os.path.join(dirname, basename + "-cert.pem"), "wb")
     f.write(OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, ca))
     f.close()
 
     # Create a .cer file with the same contents for Android
-    f = open(os.path.join(dirname, basename + "-cert.cer"), "w")
+    f = open(os.path.join(dirname, basename + "-cert.cer"), "wb")
     f.write(OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, ca))
     f.close()
 
     # Dump the certificate in PKCS12 format for Windows devices
-    f = open(os.path.join(dirname, basename + "-cert.p12"), "w")
+    f = open(os.path.join(dirname, basename + "-cert.p12"), "wb")
     p12 = OpenSSL.crypto.PKCS12()
     p12.set_certificate(ca)
     p12.set_privatekey(key)
@@ -73,7 +76,7 @@ def dummy_ca(path):
     return True
 
 
-def dummy_cert(fp, ca, commonname, sans):
+def dummy_cert(ca, commonname, sans):
     """
         Generates and writes a certificate to fp.
 
@@ -88,90 +91,49 @@ def dummy_cert(fp, ca, commonname, sans):
         ss.append("DNS: %s"%i)
     ss = ", ".join(ss)
 
-    raw = file(ca, "r").read()
+    raw = file(ca, "rb").read()
     ca = OpenSSL.crypto.load_certificate(OpenSSL.crypto.FILETYPE_PEM, raw)
     key = OpenSSL.crypto.load_privatekey(OpenSSL.crypto.FILETYPE_PEM, raw)
 
-    req = OpenSSL.crypto.X509Req()
-    subj = req.get_subject()
-    subj.CN = commonname
-    req.set_pubkey(ca.get_pubkey())
-    req.sign(key, "sha1")
-    if ss:
-        req.add_extensions([OpenSSL.crypto.X509Extension("subjectAltName", True, ss)])
-
     cert = OpenSSL.crypto.X509()
-    cert.gmtime_adj_notBefore(-3600)
+    cert.gmtime_adj_notBefore(-3600*48)
     cert.gmtime_adj_notAfter(60 * 60 * 24 * 30)
     cert.set_issuer(ca.get_subject())
-    cert.set_subject(req.get_subject())
+    cert.get_subject().CN = commonname
     cert.set_serial_number(int(time.time()*10000))
     if ss:
         cert.set_version(2)
         cert.add_extensions([OpenSSL.crypto.X509Extension("subjectAltName", True, ss)])
-    cert.set_pubkey(req.get_pubkey())
+    cert.set_pubkey(ca.get_pubkey())
     cert.sign(key, "sha1")
-
-    fp.write(OpenSSL.crypto.dump_certificate(OpenSSL.crypto.FILETYPE_PEM, cert))
-    fp.close()
+    return SSLCert(cert)
 
 
 class CertStore:
     """
-        Implements an on-disk certificate store.
+        Implements an in-memory certificate store.
     """
-    def __init__(self, certdir=None):
-        """
-            certdir: The certificate store directory. If None, a temporary
-            directory will be created, and destroyed when the .cleanup() method
-            is called.
-        """
-        if certdir:
-            self.remove = False
-            self.certdir = certdir
-        else:
-            self.remove = True
-            self.certdir = tempfile.mkdtemp(prefix="certstore")
+    def __init__(self):
+        self.certs = {}
 
-    def check_domain(self, commonname):
-        try:
-            commonname.decode("idna")
-            commonname.decode("ascii")
-        except:
-            return False
-        if ".." in commonname:
-            return False
-        if "/" in commonname:
-            return False
-        return True
-
-    def get_cert(self, commonname, sans, cacert=False):
+    def get_cert(self, commonname, sans, cacert):
         """
-            Returns the path to a certificate.
+            Returns an SSLCert object.
 
             commonname: Common name for the generated certificate. Must be a
             valid, plain-ASCII, IDNA-encoded domain name.
 
             sans: A list of Subject Alternate Names.
 
-            cacert: An optional path to a CA certificate. If specified, the
-            cert is created if it does not exist, else return None.
+            cacert: The path to a CA certificate.
 
             Return None if the certificate could not be found or generated.
         """
-        if not self.check_domain(commonname):
-            return None
-        certpath = os.path.join(self.certdir, commonname + ".pem")
-        if os.path.exists(certpath):
-            return certpath
-        elif cacert:
-            f = open(certpath, "w")
-            dummy_cert(f, cacert, commonname, sans)
-            return certpath
-
-    def cleanup(self):
-        if self.remove:
-            shutil.rmtree(self.certdir)
+        if commonname in self.certs:
+            return self.certs[commonname]
+        c = dummy_cert(cacert, commonname, sans)
+        self.certs[commonname] = c
+        return c
 
 
 class _GeneralName(univ.Choice):
