@@ -14,7 +14,10 @@ OP_DONT_INSERT_EMPTY_FRAGMENTS = SSL.OP_DONT_INSERT_EMPTY_FRAGMENTS
 OP_EPHEMERAL_RSA = SSL.OP_EPHEMERAL_RSA
 OP_MICROSOFT_BIG_SSLV3_BUFFER = SSL.OP_MICROSOFT_BIG_SSLV3_BUFFER
 OP_MICROSOFT_SESS_ID_BUG = SSL.OP_MICROSOFT_SESS_ID_BUG
-OP_MSIE_SSLV2_RSA_PADDING = SSL.OP_MSIE_SSLV2_RSA_PADDING
+try:
+    OP_MSIE_SSLV2_RSA_PADDING = SSL.OP_MSIE_SSLV2_RSA_PADDING
+except AttributeError:
+    pass
 OP_NETSCAPE_CA_DN_BUG = SSL.OP_NETSCAPE_CA_DN_BUG
 OP_NETSCAPE_CHALLENGE_BUG = SSL.OP_NETSCAPE_CHALLENGE_BUG
 OP_NETSCAPE_DEMO_CIPHER_CHANGE_BUG = SSL.OP_NETSCAPE_DEMO_CIPHER_CHANGE_BUG
@@ -37,6 +40,7 @@ OP_TLS_ROLLBACK_BUG = SSL.OP_TLS_ROLLBACK_BUG
 class NetLibError(Exception): pass
 class NetLibDisconnect(NetLibError): pass
 class NetLibTimeout(NetLibError): pass
+class NetLibSSLError(NetLibError): pass
 
 
 class _FileLike:
@@ -93,7 +97,7 @@ class Writer(_FileLike):
         if hasattr(self.o, "flush"):
             try:
                 self.o.flush()
-            except socket.error, v:
+            except (socket.error, IOError), v:
                 raise NetLibDisconnect(str(v))
 
     def write(self, v):
@@ -141,6 +145,8 @@ class Reader(_FileLike):
                 raise NetLibDisconnect
             except SSL.SysCallError:
                 raise NetLibDisconnect
+            except SSL.Error, v:
+                raise NetLibSSLError(v.message)
             self.first_byte_timestamp = self.first_byte_timestamp or time.time()
             if not data:
                 break
@@ -173,8 +179,10 @@ class Reader(_FileLike):
 class TCPClient:
     rbufsize = -1
     wbufsize = -1
-    def __init__(self, host, port):
+    def __init__(self, host, port, source_address=None, use_ipv6=False):
         self.host, self.port = host, port
+        self.source_address = source_address
+        self.use_ipv6 = use_ipv6
         self.connection, self.rfile, self.wfile = None, None, None
         self.cert = None
         self.ssl_established = False
@@ -207,12 +215,13 @@ class TCPClient:
 
     def connect(self):
         try:
-            addr = socket.gethostbyname(self.host)
-            connection = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            connection.connect((addr, self.port))
+            connection = socket.socket(socket.AF_INET6 if self.use_ipv6 else socket.AF_INET, socket.SOCK_STREAM)
+            if self.source_address:
+                connection.bind(self.source_address)
+            connection.connect((self.host, self.port))
             self.rfile = Reader(connection.makefile('rb', self.rbufsize))
             self.wfile = Writer(connection.makefile('wb', self.wbufsize))
-        except socket.error, err:
+        except (socket.error, IOError), err:
             raise NetLibError('Error connecting to "%s": %s' % (self.host, err))
         self.connection = connection
 
@@ -229,10 +238,15 @@ class TCPClient:
         try:
             if self.ssl_established:
                 self.connection.shutdown()
+                self.connection.sock_shutdown(socket.SHUT_WR)
             else:
-                self.connection.shutdown(socket.SHUT_RDWR)
+                self.connection.shutdown(socket.SHUT_WR)
+            #Section 4.2.2.13 of RFC 1122 tells us that a close() with any pending readable data could lead to an immediate RST being sent.
+            #http://ia600609.us.archive.org/22/items/TheUltimateSo_lingerPageOrWhyIsMyTcpNotReliable/the-ultimate-so_linger-page-or-why-is-my-tcp-not-reliable.html
+            while self.connection.recv(4096):
+                pass
             self.connection.close()
-        except (socket.error, SSL.Error):
+        except (socket.error, SSL.Error, IOError):
             # Socket probably already closed
             pass
 
@@ -240,7 +254,7 @@ class TCPClient:
 class BaseHandler:
     """
         The instantiator is expected to call the handle() and finish() methods.
-            
+
     """
     rbufsize = -1
     wbufsize = -1
@@ -256,8 +270,9 @@ class BaseHandler:
 
         self.clientcert = None
 
-    def convert_to_ssl(self, cert, key, method=SSLv23_METHOD, options=None, handle_sni=None, request_client_cert=False):
+    def convert_to_ssl(self, cert, key, method=SSLv23_METHOD, options=None, handle_sni=None, request_client_cert=False, cipher_list=None):
         """
+            cert: A certutils.SSLCert object.
             method: One of SSLv2_METHOD, SSLv3_METHOD, SSLv23_METHOD, or TLSv1_METHOD
             handle_sni: SNI handler, should take a connection object. Server
             name can be retrieved like this:
@@ -283,14 +298,18 @@ class BaseHandler:
         ctx = SSL.Context(method)
         if not options is None:
             ctx.set_options(options)
+        if cipher_list:
+            ctx.set_cipher_list(cipher_list)
         if handle_sni:
             # SNI callback happens during do_handshake()
             ctx.set_tlsext_servername_callback(handle_sni)
         ctx.use_privatekey_file(key)
-        ctx.use_certificate_file(cert)
+        ctx.use_certificate(cert.x509)
         if request_client_cert:
             def ver(*args):
                 self.clientcert = certutils.SSLCert(args[1])
+                # Return true to prevent cert verification error
+                return True
             ctx.set_verify(SSL.VERIFY_PEER, ver)
         self.connection = SSL.Connection(ctx, self.connection)
         self.ssl_established = True
@@ -327,8 +346,14 @@ class BaseHandler:
         try:
             if self.ssl_established:
                 self.connection.shutdown()
+                self.connection.sock_shutdown(socket.SHUT_WR)
             else:
-                self.connection.shutdown(socket.SHUT_RDWR)
+                self.connection.shutdown(socket.SHUT_WR)
+            # Section 4.2.2.13 of RFC 1122 tells us that a close() with any
+            # pending readable data could lead to an immediate RST being sent.
+            # http://ia600609.us.archive.org/22/items/TheUltimateSo_lingerPageOrWhyIsMyTcpNotReliable/the-ultimate-so_linger-page-or-why-is-my-tcp-not-reliable.html
+            while self.connection.recv(4096):
+                pass
         except (socket.error, SSL.Error):
             # Socket probably already closed
             pass
@@ -337,11 +362,12 @@ class BaseHandler:
 
 class TCPServer:
     request_queue_size = 20
-    def __init__(self, server_address):
+    def __init__(self, server_address, use_ipv6=False):
         self.server_address = server_address
+        self.use_ipv6 = use_ipv6
         self.__is_shut_down = threading.Event()
         self.__shutdown_request = False
-        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.socket = socket.socket(socket.AF_INET6 if self.use_ipv6 else socket.AF_INET, socket.SOCK_STREAM)
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self.socket.bind(self.server_address)
         self.server_address = self.socket.getsockname()
@@ -360,7 +386,13 @@ class TCPServer:
         self.__is_shut_down.clear()
         try:
             while not self.__shutdown_request:
-                r, w, e = select.select([self.socket], [], [], poll_interval)
+                try:
+                    r, w, e = select.select([self.socket], [], [], poll_interval)
+                except select.error, ex:
+                        if ex[0] == 4:
+                            continue
+                        else:
+                            raise  
                 if self.socket in r:
                     request, client_address = self.socket.accept()
                     t = threading.Thread(
