@@ -1,38 +1,8 @@
-import cStringIO, threading, Queue, time
-from netlib import tcp, certutils
+import cStringIO, Queue, time, socket, random
+from netlib import tcp, certutils, test
+import mock
 import tutils
 
-class ServerThread(threading.Thread):
-    def __init__(self, server):
-        self.server = server
-        threading.Thread.__init__(self)
-
-    def run(self):
-        self.server.serve_forever()
-
-    def shutdown(self):
-        self.server.shutdown()
-
-
-class ServerTestBase:
-    @classmethod
-    def setupAll(cls):
-        cls.q = Queue.Queue()
-        s = cls.makeserver()
-        cls.port = s.port
-        cls.server = ServerThread(s)
-        cls.server.start()
-
-    @classmethod
-    def teardownAll(cls):
-        cls.server.shutdown()
-
-
-    @property
-    def last_handler(self):
-        return self.server.server.last_handler
-
-
 class SNIHandler(tcp.BaseHandler):
     sni = None
     def handle_sni(self, connection):
@@ -50,10 +20,31 @@ class EchoHandler(tcp.BaseHandler):
 
     def handle(self):
         v = self.rfile.readline()
-        if v.startswith("echo"):
-            self.wfile.write(v)
-        elif v.startswith("error"):
-            raise ValueError("Testing an error.")
+        self.wfile.write(v)
+        self.wfile.flush()
+
+
+class ClientPeernameHandler(tcp.BaseHandler):
+    def handle(self):
+        self.wfile.write(str(self.connection.getpeername()))
+        self.wfile.flush()
+
+
+class CertHandler(tcp.BaseHandler):
+    sni = None
+    def handle_sni(self, connection):
+        self.sni = connection.get_servername()
+
+    def handle(self):
+        self.wfile.write("%s\n"%self.clientcert.serial)
+        self.wfile.flush()
+
+
+class ClientCipherListHandler(tcp.BaseHandler):
+    sni = None
+
+    def handle(self):
+        self.wfile.write("%s"%self.connection.get_cipher_list())
         self.wfile.flush()
 
 
@@ -78,76 +69,93 @@ class TimeoutHandler(tcp.BaseHandler):
             self.timeout = True
 
 
-class TServer(tcp.TCPServer):
-    def __init__(self, addr, ssl, q, handler_klass, v3_only=False):
-        tcp.TCPServer.__init__(self, addr)
-        self.ssl, self.q = ssl, q
-        self.v3_only = v3_only
-        self.handler_klass = handler_klass
-        self.last_handler = None
+class TestServer(test.ServerTestBase):
+    handler = EchoHandler
+    def test_echo(self):
+        testval = "echo!\n"
+        c = tcp.TCPClient("127.0.0.1", self.port)
+        c.connect()
+        c.wfile.write(testval)
+        c.wfile.flush()
+        assert c.rfile.readline() == testval
 
-    def handle_connection(self, request, client_address):
-        h = self.handler_klass(request, client_address, self)
-        self.last_handler = h
-        if self.ssl:
-            if self.v3_only:
-                method = tcp.SSLv3_METHOD
-                options = tcp.OP_NO_SSLv2|tcp.OP_NO_TLSv1
-            else:
-                method = tcp.SSLv23_METHOD
-                options = None
-            h.convert_to_ssl(
-                tutils.test_data.path("data/server.crt"),
-                tutils.test_data.path("data/server.key"),
-                method = method,
-                options = options,
+
+class TestServerBind(test.ServerTestBase):
+    handler = ClientPeernameHandler
+
+    def test_bind(self):
+        """ Test to bind to a given random port. Try again if the random port turned out to be blocked. """
+        for i in range(20):
+            random_port = random.randrange(1024, 65535)
+            try:
+                c = tcp.TCPClient("127.0.0.1", self.port, source_address=("127.0.0.1", random_port))
+                c.connect()
+                assert c.rfile.readline() == str(("127.0.0.1", random_port))
+                return
+            except tcp.NetLibError: # port probably already in use
+                pass
+
+
+class TestServerIPv6(test.ServerTestBase):
+    handler = EchoHandler
+    use_ipv6 = True
+
+    def test_echo(self):
+        testval = "echo!\n"
+        c = tcp.TCPClient("::1", self.port, use_ipv6=True)
+        c.connect()
+        c.wfile.write(testval)
+        c.wfile.flush()
+        assert c.rfile.readline() == testval
+
+
+class FinishFailHandler(tcp.BaseHandler):
+    def handle(self):
+        v = self.rfile.readline()
+        self.wfile.write(v)
+        self.wfile.flush()
+        self.wfile.close()
+        self.rfile.close()
+        self.close = mock.MagicMock(side_effect=socket.error)
+
+
+class TestFinishFail(test.ServerTestBase):
+    """
+        This tests a difficult-to-trigger exception in the .finish() method of
+        the handler.
+    """
+    handler = FinishFailHandler
+    def test_disconnect_in_finish(self):
+        testval = "echo!\n"
+        c = tcp.TCPClient("127.0.0.1", self.port)
+        c.connect()
+        c.wfile.write("foo\n")
+        c.wfile.flush()
+        c.rfile.read(4)
+
+class TestDisconnect(test.ServerTestBase):
+    handler = EchoHandler
+    def test_echo(self):
+        testval = "echo!\n"
+        c = tcp.TCPClient("127.0.0.1", self.port)
+        c.connect()
+        c.wfile.write(testval)
+        c.wfile.flush()
+        assert c.rfile.readline() == testval
+
+
+class TestServerSSL(test.ServerTestBase):
+    handler = EchoHandler
+    ssl = dict(
+                cert = tutils.test_data.path("data/server.crt"),
+                key = tutils.test_data.path("data/server.key"),
+                request_client_cert = False,
+                v3_only = False
             )
-        h.handle()
-        h.finish()
-
-    def handle_error(self, request, client_address):
-        s = cStringIO.StringIO()
-        tcp.TCPServer.handle_error(self, request, client_address, s)
-        self.q.put(s.getvalue())
-
-
-class TestServer(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), False, cls.q, EchoHandler)
-
-    def test_echo(self):
-        testval = "echo!\n"
-        c = tcp.TCPClient("127.0.0.1", self.port)
-        c.connect()
-        c.wfile.write(testval)
-        c.wfile.flush()
-        assert c.rfile.readline() == testval
-
-
-class TestDisconnect(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), False, cls.q, EchoHandler)
-
-    def test_echo(self):
-        testval = "echo!\n"
-        c = tcp.TCPClient("127.0.0.1", self.port)
-        c.connect()
-        c.wfile.write(testval)
-        c.wfile.flush()
-        assert c.rfile.readline() == testval
-
-
-class TestServerSSL(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), True, cls.q, EchoHandler)
-
     def test_echo(self):
         c = tcp.TCPClient("127.0.0.1", self.port)
         c.connect()
-        c.convert_to_ssl(sni="foo.com")
+        c.convert_to_ssl(sni="foo.com", options=tcp.OP_ALL)
         testval = "echo!\n"
         c.wfile.write(testval)
         c.wfile.flush()
@@ -157,22 +165,52 @@ class TestServerSSL(ServerTestBase):
         assert certutils.get_remote_cert("127.0.0.1", self.port, None).digest("sha1")
 
 
-class TestSSLv3Only(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), True, cls.q, EchoHandler, True)
-
+class TestSSLv3Only(test.ServerTestBase):
+    handler = EchoHandler
+    ssl = dict(
+        cert = tutils.test_data.path("data/server.crt"),
+        key = tutils.test_data.path("data/server.key"),
+        request_client_cert = False,
+        v3_only = True
+    )
     def test_failure(self):
         c = tcp.TCPClient("127.0.0.1", self.port)
         c.connect()
         tutils.raises(tcp.NetLibError, c.convert_to_ssl, sni="foo.com", method=tcp.TLSv1_METHOD)
 
 
-class TestSNI(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), True, cls.q, SNIHandler)
+class TestSSLClientCert(test.ServerTestBase):
+    handler = CertHandler
+    ssl = dict(
+        cert = tutils.test_data.path("data/server.crt"),
+        key = tutils.test_data.path("data/server.key"),
+        request_client_cert = True,
+        v3_only = False
+    )
+    def test_clientcert(self):
+        c = tcp.TCPClient("127.0.0.1", self.port)
+        c.connect()
+        c.convert_to_ssl(cert=tutils.test_data.path("data/clientcert/client.pem"))
+        assert c.rfile.readline().strip() == "1"
 
+    def test_clientcert_err(self):
+        c = tcp.TCPClient("127.0.0.1", self.port)
+        c.connect()
+        tutils.raises(
+            tcp.NetLibError,
+            c.convert_to_ssl,
+            cert=tutils.test_data.path("data/clientcert/make")
+        )
+
+
+class TestSNI(test.ServerTestBase):
+    handler = SNIHandler
+    ssl = dict(
+        cert = tutils.test_data.path("data/server.crt"),
+        key = tutils.test_data.path("data/server.key"),
+        request_client_cert = False,
+        v3_only = False
+    )
     def test_echo(self):
         c = tcp.TCPClient("127.0.0.1", self.port)
         c.connect()
@@ -180,11 +218,30 @@ class TestSNI(ServerTestBase):
         assert c.rfile.readline() == "foo.com"
 
 
-class TestSSLDisconnect(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), True, cls.q, DisconnectHandler)
+class TestClientCipherList(test.ServerTestBase):
+    handler = ClientCipherListHandler
+    ssl = dict(
+        cert = tutils.test_data.path("data/server.crt"),
+        key = tutils.test_data.path("data/server.key"),
+        request_client_cert = False,
+        v3_only = False,
+        cipher_list = 'RC4-SHA'
+    )
+    def test_echo(self):
+        c = tcp.TCPClient("127.0.0.1", self.port)
+        c.connect()
+        c.convert_to_ssl(sni="foo.com")
+        assert c.rfile.readline() == "['RC4-SHA']"
 
+
+class TestSSLDisconnect(test.ServerTestBase):
+    handler = DisconnectHandler
+    ssl = dict(
+        cert = tutils.test_data.path("data/server.crt"),
+        key = tutils.test_data.path("data/server.key"),
+        request_client_cert = False,
+        v3_only = False
+    )
     def test_echo(self):
         c = tcp.TCPClient("127.0.0.1", self.port)
         c.connect()
@@ -196,26 +253,18 @@ class TestSSLDisconnect(ServerTestBase):
         tutils.raises(Queue.Empty, self.q.get_nowait)
 
 
-class TestDisconnect(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), False, cls.q, DisconnectHandler)
-
+class TestDisconnect(test.ServerTestBase):
     def test_echo(self):
         c = tcp.TCPClient("127.0.0.1", self.port)
         c.connect()
-        # Excercise SSL.ZeroReturnError
         c.rfile.read(10)
         c.wfile.write("foo")
         c.close()
         c.close()
 
 
-class TestServerTimeOut(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), False, cls.q, TimeoutHandler)
-
+class TestServerTimeOut(test.ServerTestBase):
+    handler = TimeoutHandler
     def test_timeout(self):
         c = tcp.TCPClient("127.0.0.1", self.port)
         c.connect()
@@ -223,23 +272,24 @@ class TestServerTimeOut(ServerTestBase):
         assert self.last_handler.timeout
 
 
-class TestTimeOut(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), False, cls.q, HangHandler)
-
+class TestTimeOut(test.ServerTestBase):
+    handler = HangHandler
     def test_timeout(self):
         c = tcp.TCPClient("127.0.0.1", self.port)
         c.connect()
         c.settimeout(0.1)
+        assert c.gettimeout() == 0.1
         tutils.raises(tcp.NetLibTimeout, c.rfile.read, 10)
 
 
-class TestSSLTimeOut(ServerTestBase):
-    @classmethod
-    def makeserver(cls):
-        return TServer(("127.0.0.1", 0), True, cls.q, HangHandler)
-
+class TestSSLTimeOut(test.ServerTestBase):
+    handler = HangHandler
+    ssl = dict(
+        cert = tutils.test_data.path("data/server.crt"),
+        key = tutils.test_data.path("data/server.key"),
+        request_client_cert = False,
+        v3_only = False
+    )
     def test_timeout_client(self):
         c = tcp.TCPClient("127.0.0.1", self.port)
         c.connect()
@@ -313,3 +363,44 @@ class TestFileLike:
         s.write("x")
         assert s.get_log() == "xx"
 
+    def test_writer_flush_error(self):
+        s = cStringIO.StringIO()
+        s = tcp.Writer(s)
+        o = mock.MagicMock()
+        o.flush = mock.MagicMock(side_effect=socket.error)
+        s.o = o
+        tutils.raises(tcp.NetLibDisconnect, s.flush)
+
+    def test_reader_read_error(self):
+        s = cStringIO.StringIO("foobar\nfoobar")
+        s = tcp.Reader(s)
+        o = mock.MagicMock()
+        o.read = mock.MagicMock(side_effect=socket.error)
+        s.o = o
+        tutils.raises(tcp.NetLibDisconnect, s.read, 10)
+
+    def test_reset_timestamps(self):
+        s = cStringIO.StringIO("foobar\nfoobar")
+        s = tcp.Reader(s)
+        s.first_byte_timestamp = 500
+        s.reset_timestamps()
+        assert not s.first_byte_timestamp
+
+    def test_first_byte_timestamp_updated_on_read(self):
+        s = cStringIO.StringIO("foobar\nfoobar")
+        s = tcp.Reader(s)
+        s.read(1)
+        assert s.first_byte_timestamp
+        expected = s.first_byte_timestamp
+        s.read(5)
+        assert s.first_byte_timestamp == expected
+
+    def test_first_byte_timestamp_updated_on_readline(self):
+        s = cStringIO.StringIO("foobar\nfoobar\nfoobar")
+        s = tcp.Reader(s)
+        s.readline()
+        assert s.first_byte_timestamp
+        expected = s.first_byte_timestamp
+        s.readline()
+        assert s.first_byte_timestamp == expected
+
